@@ -9,11 +9,13 @@ use Carp;
 require DynaLoader;
 our @ISA = qw(DynaLoader);
 
-our $VERSION = 1.10;
+our $VERSION = 1.11;
 
 bootstrap Math::Random::MT::Auto $VERSION;
 
-# Default seeding methods
+### Global Variables ###
+
+# Default seeding sources
 my @seeders;
 
 # Auto-seed the PRNG via an INIT block
@@ -26,49 +28,9 @@ my @seed;
 our @errors;
 
 
-# Handles importation of random functions,
-#   and specification of seeding methods
-sub import
-{
-    my $class = shift;
-    my $pkg = caller;
-    no strict 'refs';
+### Initializations ###
 
-    my @methods;
-
-    while (my $sym = shift) {
-        # Exportable functions
-        if ($sym eq 'rand32' ||
-            $sym eq 'rand'   ||
-            $sym eq 'srand'  ||
-            $sym eq 'seed'   ||
-            $sym eq 'state')
-        {
-            *{"${pkg}::$sym"} = \&$sym;
-
-        } elsif ($sym =~ 'auto') {
-            # To auto-seed or not
-            $auto_seed = $sym =~ /no|!/;
-
-        } else {
-            # User-specified seed acquisition methods
-            # or user-defined seed acquisition functions
-            push(@methods, $sym);
-            # Max. count for method, if any
-            if (@_ && looks_like_number($_[0])) {
-                push(@methods, shift(@_));
-            }
-        }
-    }
-
-    # Save user-specified seed acquisition methods
-    if (@methods) {
-        @seeders = @methods;
-    }
-}
-
-
-# Set up default seed sources
+# 1. Set up default seed sources.
 BEGIN {
     if (-e '/dev/urandom') {
         push(@seeders, '/dev/urandom');
@@ -79,47 +41,97 @@ BEGIN {
 }
 
 
-# Auto seed the PRNG when module is loaded
-# Even when $auto_seed is false, the PRNG is still seeded using time and PID
-INIT {
-    Math::Random::MT::Auto::_seed(($auto_seed) ? @seeders : 'none');
+# 2. Handles importation of random functions,
+# and specification of seeding sources by user.
+sub import
+{
+    my $class = shift;
+    my $pkg = caller;
+
+    my @sources;
+
+    while (my $sym = shift) {
+        # Exportable functions
+        if ($sym eq 'rand32' ||
+            $sym eq 'rand'   ||
+            $sym eq 'srand'  ||
+            $sym eq 'seed'   ||
+            $sym eq 'state')
+        {
+            no strict 'refs';
+            *{"${pkg}::$sym"} = \&$sym;
+
+        } elsif ($sym =~ /(no|!)?auto/) {
+            # To auto-seed or not
+            $auto_seed = not defined($1);
+
+        } else {
+            # User-specified seed acquisition sources
+            # or user-defined seed acquisition functions
+            push(@sources, $sym);
+            # Max. count for source, if any
+            if (@_ && looks_like_number($_[0])) {
+                push(@sources, shift(@_));
+            }
+        }
+    }
+
+    # Save user-specified seed acquisition sources
+    if (@sources) {
+        @seeders = @sources;
+    }
 }
 
 
-# Starts PRNG with random seed using specified methods (if any)
+# 3. Auto seed the PRNG after the module is loaded.
+# Even when $auto_seed is false, the PRNG is still
+# seeded using time and PID.
+{
+    no warnings;
+    INIT {
+        _seeder(($auto_seed) ? @seeders : 'none');
+    }
+}
+
+
+### Exportable Subroutines ###
+
+# Starts PRNG with random seed using specified sources (if any)
 sub srand
 {
-    # Save specified methods, if any
     if (@_) {
+        # Check if sent seeds by mistake
+        if (looks_like_number($_[0]) || ref($_[0]) eq 'ARRAY') {
+            seed(@_);
+        }
+
+        # Save specified sources
         @seeders = @_;
     }
 
     # Seed the PRNG
-    _seed(@seeders);
+    _seeder(@seeders);
 }
 
 
 # Use supplied seed, if given
-# Returns ref to saved seed
+# Returns ref to saved seed if no args
 sub seed
 {
-    my $seed = $_[0];
-
     # User requested the seed
-    if (! defined($seed)) {
+    if (! @_) {
         return (\@seed);
     }
 
-    # Check input
-    if (ref($seed) ne 'ARRAY') {
-        croak('Arg to seed() not an array ref');
+    # Save a copy of the seed
+    if (ref($_[0]) eq 'ARRAY') {
+        @seed = @{$_[0]};
+    } else {
+        @seed = @_;
     }
 
-    # Save a copy of the seed
-    @seed = @{$seed};
-
     # Seed the PRNG
-    Math::Random::MT::Auto::_seed_prng(\@seed);
+    _init(\@seed);
 }
 
 
@@ -131,57 +143,66 @@ sub state
 
     # User requested copy of state
     if (! defined($state)) {
-        return (Math::Random::MT::Auto::_get_state());
+        return (_get_state());
     }
 
     # Set state of PRNG
-    Math::Random::MT::Auto::_set_state($state);
+    _set_state($state);
 }
 
 
 ### Internal Subroutines ###
 
+my %_seeder_dispatch = (
+    'random_org' => \&_seeder_random_org,
+    'hotbits'    => \&_seeder_hotbits
+);
+
 # Seeds the PRNG
-sub _seed
+sub _seeder
 {
-    my @methods = @_;
+    my @sources = @_;
 
     @seed = ();
+    my $FULL_SEED = 624;
 
-    for (my $ii=0; $ii < @methods; $ii++) {
-        my $method = $methods[$ii];
+    for (my $ii=0; $ii < @sources; $ii++) {
+        my $source = $sources[$ii];
 
-        my $need = 624 - @seed;
-        if (($ii+1 < @methods) && looks_like_number($methods[$ii+1])) {
-            if ($methods[++$ii] < $need) {
-                $need = $methods[$ii];
+        # Determine amount of data needed
+        my $need = $FULL_SEED - @seed;
+        if (($ii+1 < @sources) && looks_like_number($sources[$ii+1])) {
+            if ($sources[++$ii] < $need) {
+                $need = $sources[$ii];
             }
         }
 
-        if (ref($method) eq 'CODE') {
+        if (ref($source) eq 'CODE') {
             # User supplied seeding function
-            &$method(\@seed, $need);
+            &$source(\@seed, $need);
 
-        } elsif ($method =~ /[\/\\]/) {
-            # Random device
-            _seed_dev($method, \@seed, $need);
+        } elsif ($source =~ /[\/\\]/) {
+            # Random device or file
+            _seeder_dev($source, \@seed, $need);
 
-        } elsif ($method ne 'none') {
-            # Module defined seeding method
-            #   _seed_random_org, ...
-            no strict 'refs';
-            &{"_seed_$method"}(\@seed, $need);
+        } elsif ($source ne 'none') {
+            # Module defined seeding source
+            if (defined($_seeder_dispatch{$source})) {
+                $_seeder_dispatch{$source}(\@seed, $need);
+            } else {
+                push(@errors, "Unknown seeding source: $source");
+            }
         }
 
         # Check if done
-        if (@seed >= 624) {
+        if (@seed >= $FULL_SEED) {
             last;
         }
     }
 
     # If still needed, make use of time and PID
-    if (@seed < 624) {
-        if ($methods[0] ne 'none') {
+    if (@seed < $FULL_SEED) {
+        if ($sources[0] ne 'none') {
             push(@errors, 'Full seed not acquired from sources: ' . scalar(@seed));
         }
         if (! @seed) {
@@ -189,13 +210,12 @@ sub _seed
         }
     }
 
-
     # Seed the PRNG
-    Math::Random::MT::Auto::_seed_prng(\@seed);
+    _init(\@seed);
 }
 
 
-sub _seed_dev
+sub _seeder_dev
 {
     my $device = $_[0];
     my $seed   = $_[1];
@@ -247,14 +267,17 @@ sub _seed_dev
 }
 
 
-sub _seed_random_org
+sub _seeder_random_org
 {
     my $seed = $_[0];
     my $need = $_[1];
     my $bytes  = 4 * $need;
 
     # Get data from random.org
-    eval { require LWP::UserAgent; };
+    eval {
+        require LWP::UserAgent;
+        import LWP::UserAgent;
+    };
     if ($@) {
         push(@errors, "Failure importing LWP::UserAgent: $@");
         return;
@@ -281,14 +304,17 @@ sub _seed_random_org
 }
 
 
-sub _seed_hotbits
+sub _seeder_hotbits
 {
     my $seed = $_[0];
     my $need = $_[1];
     my $bytes  = 4 * $need;
 
     # Get data from random.org
-    eval { require LWP::UserAgent; };
+    eval {
+        require LWP::UserAgent;
+        import LWP::UserAgent;
+    };
     if ($@) {
         push(@errors, "Failure importing LWP::UserAgent: $@");
         return;
@@ -343,14 +369,40 @@ Math::Random::MT::Auto - Auto-seeded Mersenne Twister PRNG
 
 =head1 DESCRIPTION
 
-This module provides two random number functions that are based on the
-Mersenne Twister pseudorandom number generator (PRNG).
+The Mersenne Twister is a fast pseudo-random number generator (PRNG) that
+is capable of providing large volumes (> 2.5 * 10^6004) of "high quality"
+pseudo-random data to applications that may exhaust available "truly"
+random data source or system-provided PRNGs such as C<rand>.
 
-The PRNG is self-seeding, automatically acquiring a 624-long-int random
+This module provides two random number functions (L</"rand"> and
+L</"rand32">) that are based on the Mersenne Twister.  Additionally, the
+PRNG is self-seeding, automatically acquiring a 624-long-integer random
 seed when the module is loaded.
 
+The design philosophy for this module emphasizes speed and simplicity.
+(Hence, no OO interface which compromises both, as well as defeating the
+auto-seeding feature of this module.)
+
+=head2 Quickstart
+
+To use this module as a drop-in replacement for Perl's C<rand> function,
+just add the following to the top of your application code:
+
+  use Math::Random::MT::Auto qw/rand/;
+
+and then just use C<rand> as you would normally.  You don't even need to
+bother seeding the PRNG (i.e., you don't need to use C<srand>), as that
+gets done automatically when the module is loaded by Perl.
+
+B<CAUTION>: If you want to C<require> this module, see the
+L</"Delayed Importation"> section for important information.
+
 =head2 Seeding Sources
 
+Starting the PRNG with a 624-long-integer random seed takes advantage of
+the PRNG's full range of possible internal vectors states.  This module
+attempts to acquire such a seed using several user-selectable sources.
+
 =over
 
 =item Random Devices
@@ -403,8 +455,8 @@ the C<hotbits> source twice in the module declaration.
 
 The default list of seeding sources is determined in a C<BEGIN> block
 when the module is loaded.  First, F</dev/urandom> and then
-F</dev/random> are checked.  The first one found is added to the list.
-Then C<random_org> is added.
+F</dev/random> are checked.  The first one found is added to the list,
+and then C<random_org> is added.
 
 These defaults can be overriden by specifying the desired sources when
 the module is declared, or through the use of the L</"srand"> function.
@@ -412,8 +464,8 @@ the module is declared, or through the use of the L</"srand"> function.
 Optionally, the maximum number of long-ints to be used from a source
 may be specified.
 
-  # Get at most 500 long-ints from random.org, and then
-  #  finishes the rest of the seed using data from /dev/urandom
+  # Get at most 500 long-ints from random.org
+  # Finish the seed using data from /dev/urandom
   use Math::Random::MT::Auto 'random_org' => 500,
                              '/dev/urandom';
 
@@ -422,26 +474,11 @@ could easily be included in future versions of this module.)
 
 =back
 
-=head2 Auto-seeding
-
-Normally, the PRNG is seeded in an C<INIT> block that is executed right
-after the module is loaded.  This behavior can be modified by supplying the
-C<:!auto> (or C<:noauto>) flag when the module is declared.  (Even in this
-case, the PRNG will still be seeded using time and PID.)  When the
-C<:!auto> option is used, the L</"srand"> function should be imported and
-then run before calling L</"rand"> or L</"rand32">.
-
-  use Math::Random::MT::Auto qw/rand srand :!auto/;
-    ...
-  srand();
-    ...
-  my $rn = rand(10);
-
 =head2 Functions
 
-By default, this module does not automatically export any of the functions
-below.  If you want to use them without the full module-path, then you must
-specify them when you declare the module.
+By default, this module does not automatically export any of its functions.
+If you want to use them without the full module-path, then you must specify
+them when you declare the module.
 
   use Math::Random::MT::Auto qw/rand rand32 srand seed state/;
 
@@ -449,8 +486,8 @@ specify them when you declare the module.
 
 =item rand($num)
 
-Behaves exactly like Perl's builtin L<perlfunc/"rand">, returning a number
-uniformly distributed in [0, $num) ($num defaults to 1).
+Behaves exactly like Perl's builtin C<rand>, returning a number uniformly
+distributed in [0, $num).  ($num defaults to 1.)
 
 =item rand32()
 
@@ -491,13 +528,17 @@ long-integers needed.
     #  then get the rest from random.org.
     srand(\&MySeeder => 256, 'random_org');
 
+If called with long-integer data (single value or an array), or an array
+reference of long-integers, these data will be passed to L</"seed"> for
+use in reseeding the PRNG.
+
 =item seed
 
 When called without arguments, this function will return an array reference
 containing the seed last sent to the PRNG.
 
-When called with an array reference, the long-integer data contained
-therein will be used to reseed the PRNG.
+When called with long-integer data (single value or an array), or an array
+reference of long-integers, these data will be used to reseed the PRNG.
 
 Together, these functions may be useful for setting up identical sequences
 of random numbers based on the same seed.
@@ -524,9 +565,9 @@ obtained state-vector array reference.
 
   # $rand1 and $rand2 will be equal.
 
-CAUTION:  It should go without saying that you should not modify the values
-in the state vector, but I'll say it anyway: "You should not modify the
-values in the state vector."  'Nough said.
+B<CAUTION>:  It should go without saying that you should not modify the
+values in the state vector, but I'll say it anyway: "You should not modify
+the values in the state vector."  'Nough said.
 
 In conjunction with L<Data::Dumper> and C<do(file)>, this function can be
 used to save and then reload the state vector between application runs.
@@ -535,6 +576,42 @@ example of this.
 
 =back
 
+=head2 Delayed Seeding
+
+Normally, the PRNG is automatically seeded when the module is loaded.
+This behavior can be modified by supplying the C<:!auto> (or C<:noauto>)
+flag when the module is declared.  (The PRNG will still be seeded using
+time and PID just in case.)  When the C<:!auto> option is used, the
+L</"srand"> function should be imported, and then run before calling
+L</"rand"> or L</"rand32">.
+
+  use Math::Random::MT::Auto qw/rand srand :!auto/;
+    ...
+  srand();
+    ...
+  my $rn = rand(10);
+
+=head2 Delayed Importation
+
+When this module is imported via C<use>, the PRNG is initialized via an
+C<INIT> block that is executed right after the module is loaded.  However,
+if you want to delay the importation of this module using C<require>, then
+you B<must> import L</"srand"> and execute it so that the PRNG gets
+initialized:
+
+  eval {
+      require Math::Random::MT::Auto;
+      # Add other symbols to the import call, as desired.
+      import Math::Random::MT::Auto qw/srand/;
+      # Add optional arguments to the srand() call, as desired.
+      srand();
+  };
+
+(Otherwise, core dump!  E<lt>Flame protection ONE<gt>  For speed
+considerations, I purposely removed the code that checked if the PRNG was
+initialized because the check would get executed with every call to get a
+random number.  E<lt>Flame protection offE<gt>)
+
 =head1 DIAGNOSTICS
 
 =over
@@ -550,6 +627,10 @@ the array if desired.
 
 =back
 
+This module sets a 10 second timeout for Internet connections so that if
+something goes awry when trying to get seed data from an Internet source,
+your application will not hang for an inordinate amount of time.
+
 If you connect to the Internet through an HTTP proxy, then you must set the
 C<http_proxy> variable in your environment when using the Internet seed
 sources.  (See L<LWP::UserAgent/"Proxy attributes">.)
@@ -569,9 +650,9 @@ the possible PRNG state vectors.
 
 =head1 PERFORMANCE
 
-This module is 50% faster than Math::Random::MT.  The file
-F<samples/random>, included in this module's distribution, can be used to
-compare timing results.
+This module is 50% faster than Math::Random::MT.  (Most of this is due to
+the elimination of the OO interface.)  The file F<samples/random>, included
+in this module's distribution, can be used to compare timing results.
 
 Depending on your connnection speed, acquiring seed data from the Internet
 may take a couple of seconds.  This delay might be apparent when your
@@ -583,10 +664,9 @@ Internet sources only as a secondary source.)
 
 =head1 SEE ALSO
 
-The Mersenne Twister is the (current) quintessential pseudorandom number
-generator. It is fast, and has a period of 2^19937 - 1 (> 10^6000).  The
-Mersenne Twister algorithm was developed by Makoto Matsumoto and Takuji
-Nishimura.
+The Mersenne Twister is the (current) quintessential pseudo-random number
+generator. It is fast, and has a period of 2^19937 - 1.  The Mersenne
+Twister algorithm was developed by Makoto Matsumoto and Takuji Nishimura.
 L<http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html>
 
 random.org generates random numbers from radio frequency noise.
